#property version   "1.00"
#property strict

#define EVENT_NAME "MARCO_EXCELTRADER"

struct ORDER_INFO {
	int id;
	char symbol[12];
	double lots;
	double sl;
	double tp;
	int trade;
	double price;
	double trail_start;
	double trail_distance;
};

enum PRICE_INFO {
	openPrice, //Open Price
	highPrice, //High Price
	lowPrice, //Low Price
	closePrice //Close Price
};

#include <stdlib.mqh>

struct ExcelInfo {
	uchar sheet[12];
	int row;
	int col;
};

struct QuoteInfo {	
	datetime time;
	double price;
};

#import "kernel32.dll"
int CreateEventW(int lpEventAttributes, int bManualReset, int bInitialState, string lpName);
int SetEvent(int hEvent);
int ResetEvent(int hEvent);
int CloseHandle(int hEvent);
uint WaitForSingleObject(int hHandle, uint dwMilliseconds);

#import "ExcelLibMarco.dll"
	int ExcelInit(uchar& book[]);
	int ReadCell(ExcelInfo& xlInfo, uchar& buffer[]);
	int SendQuote(ExcelInfo& xlInfo, QuoteInfo& quote);
	int SendQuotes(ExcelInfo& xlInfo, QuoteInfo& quotes[], int count);
#import

input string 		Workbook = "ExcelTrader_1.1.xlsm";
input int			QuoteRow = 3;
input int			QuoteColumn = 1;
input int			SignalColumn = 29; //Open Signal Column
input int			CloseColumn = 30; //Close Signal Column
input int			TPRow = 4; // Take Profit Row
input int			TPColumn = 23; // Take Profit Column
input int			SLRow = 5; // Stop Loss Row
input int			SLColumn = 23; // Stop Loss Column
input double		LotSize = 1;

input int			HistoryBars = 100;
input PRICE_INFO 	Price = closePrice;
input bool        ShowAlert = false; //Show Alert

int hEvent;

bool gStarted = false;
int Shift;

ORDER_INFO Orders[];

datetime LastBarTime;

int NextRow = 0;

double pip;

int OnInit() {
	hEvent = CreateEventW(0, 0, 0, EVENT_NAME);
	if (hEvent == 0) {
		Alert("Event error, EA terminated!");
		return INIT_FAILED;
	}
	Print("hEvent ", hEvent);
	ResetEvent(hEvent);

	uchar buffer[];
	StringToCharArray(Workbook, buffer);
	int res = ExcelInit(buffer);
	if (res != 1) {
		Alert("Excel error, EA terminated!");
		return INIT_FAILED;	
	}
	EventSetMillisecondTimer(97);
	Comment("Waiting for tick...");
	
	LastBarTime = Time[0];
	pip = Point;
	if (Digits == 3 || Digits == 5) {
		pip = 10*Point;
	}
	return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason) {
	if (hEvent != 0) {
		CloseHandle(hEvent);
		hEvent = 0;
	}
	EventKillTimer();
	Comment("");
}

void OnTick() {
	if (!gStarted) {
	   Comment("Sending historical quotes...");
		gStarted = true;
		ArrayResize(Orders, 0);
		
		QuoteInfo quotes[];
		int count;
		if (Price == openPrice) {
			count = fmin(Bars, HistoryBars);
			Shift = 0;
		}
		else {
			count = fmin(Bars-1, HistoryBars);
			Shift = 1;
		}
		
		/* 
			Q[4] - Q[0]
			Bars = 5

			count = 5
			shift = 0			
			Q[5-1] - Q[0]
			Q[4] - Q[0]
			
			count = 4
			shift = 1			
			Q[4-1+1] - Q[0+1]			
			Q[4] - Q[1]
			
		*/
		
		ArrayResize(quotes, count);
		for (int i=count-1; i>=0; i--) {
			quotes[i].time = Time[i+Shift];
			if (Price == openPrice) {
				quotes[i].price = Open[i+Shift];
			}
			else if (Price == highPrice) {
			   quotes[i].price = High[i+Shift];
			}
         else if (Price == lowPrice) {
            quotes[i].price = Low[i+Shift];
         }
			else {			   
				quotes[i].price = Close[i+Shift];
			}
		}
		
		ExcelInfo xlInfo;
		StringToCharArray(Symbol(), xlInfo.sheet);
		xlInfo.row = QuoteRow;
		xlInfo.col = 1;
		
		
		int res = SendQuotes(xlInfo, quotes, count);
		
		if (res) {
			//Alert("SendQuotes ", count, " rows");
			Print("SendQuotes ", count, " rows");
		}
		else {
		   if (ShowAlert) {
			   Alert("SendQuotes ", count, " rows FAILED!");
			}
			Print("SendQuotes ", count, " rows FAILED!");
		}
		NextRow = xlInfo.row + count;
	}
	
	bool newBar = false;
	if (Time[0] > LastBarTime) {
		LastBarTime = Time[0];
		newBar = true;
		
		Comment("Sending new quote...");

		ExcelInfo xlInfo;
		StringToCharArray(Symbol(), xlInfo.sheet);
		xlInfo.row = NextRow;
		xlInfo.col = 1;
		
		QuoteInfo quote;
		quote.time = Time[Shift];
		if (Price == openPrice) {
			quote.price = Open[Shift];
		}
		else if (Price == highPrice) {
		   quote.price = High[Shift];
		}
		else if (Price == lowPrice) {
		   quote.price = Low[Shift];
		}		
		else {
			quote.price = Close[Shift];
		}
		
		int res = SendQuote(xlInfo, quote);
		if (res) {
			NextRow++;
			uchar buffer[32];
						
			//read close signal
			xlInfo.col = CloseColumn;			
			res = ReadCell(xlInfo, buffer);
			if (res) {
				string str = CharArrayToString(buffer);
				str = StringTrimLeft(StringTrimRight(str));
				int sig = (int)StringToInteger(str);
				if (sig != 0) {		
					if (ShowAlert) {
						Alert("Close Signal ", str);
					}
					//Close all
					CloseAllOrders();										
				}
			}
						
			//read open signal
			xlInfo.col = SignalColumn;			
			res = ReadCell(xlInfo, buffer);
			if (res) {
				string str = CharArrayToString(buffer);
				str = StringTrimLeft(StringTrimRight(str));
				int sig = (int)StringToInteger(str);
				if (sig != 0) {
					if (ShowAlert) {				
						Alert("Open Signal ", str);
					}
					int count;
					double sl=0, tp=0;
					double slPip=0, tpPip=0;
					//read tp
					xlInfo.row = TPRow; xlInfo.col = TPColumn;
					res = ReadCell(xlInfo, buffer);
					if (res) {
						str = CharArrayToString(buffer);
						str = StringTrimLeft(StringTrimRight(str));
						tpPip = fmax(0, StringToDouble(str));
					}
					
					//read sl
					xlInfo.row = SLRow; xlInfo.col = SLColumn;
					res = ReadCell(xlInfo, buffer);
					if (res) {
						str = CharArrayToString(buffer);
						str = StringTrimLeft(StringTrimRight(str));
						slPip = fmax(0, StringToDouble(str));
					}					
					
					
					if (sig > 0) {
						count = sig - OrderCount(OP_BUY);
						if (count > 0) {
							for (int i=0; i<count; i++) {
								RefreshRates();
								if (tpPip > 0) tp = Ask + tpPip*pip;
								if (slPip > 0) sl = Ask - slPip*pip;
								
								if (tp > 0) {
								   tp = fmax(tp, Ask + MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);
								}
								if (sl > 0) {
								   sl = fmin(sl, Bid - MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);
								}																
								
								int ticket = OrderSend(Symbol(), OP_BUY, LotSize, Ask, 99, sl, tp, WindowExpertName(), 0, 0, clrBlue);
								if (ticket < 1) {
									if (ShowAlert) {
								   	Alert(StringFormat("Buy order failed with error %s", ErrorDescription(GetLastError())));
								   }
								   Print(StringFormat("Bid %G, sl %G, tp %G", Bid, sl, tp));
								}
							}
						}
					}
					else {
						sig = fabs(sig);
						count = sig - OrderCount(OP_SELL);
						if (count > 0) {
							for (int i=0; i<count; i++) {
								RefreshRates();
								if (tpPip > 0) tp = Bid - tpPip*pip;
								if (slPip > 0) sl = Bid + slPip*pip;
								
								if (tp > 0) {
								   tp = fmin(tp, Bid - MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);
								}
								if (sl > 0) {
								   sl = fmax(sl, Ask + MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);
								}								
								int ticket = OrderSend(Symbol(), OP_SELL, LotSize, Bid, 99, sl, tp, WindowExpertName(), 0, 0, clrRed);
								if (ticket < 1) {
									if (ShowAlert) {
								   	Alert(StringFormat("Buy order failed with error %s", ErrorDescription(GetLastError())));
								   }
								   Print(StringFormat("Bid %G, sl %G, tp %G, stoplevel %G", Bid, sl, tp, MarketInfo(Symbol(), MODE_STOPLEVEL)*Point));
								}								
							}
						}					
					}
				}
			}			
		}
		else {
			Print("SendQuote FAILED!");
		}
	}
	else {
	   Comment("Waiting for new bar...");
	}
}

void OnTimer() {

}

int OrderCount(int ot) {
	int res = 0;
	for (int i=0; i<OrdersTotal(); i++) {
		if (OrderSelect(i, SELECT_BY_POS)) {
			if (OrderSymbol() == Symbol()) {
				if (OrderType() == ot) {
					res++;
				}
			}
		}
	}
	return res;
}

void CloseAllOrders() {
	for (int i=OrdersTotal()-1; i>=0; i--) {
		if (OrderSelect(i, SELECT_BY_POS)) {
			if (OrderSymbol() == Symbol()) {
				for (int k=0; k<3; k++) {
					RefreshRates();
					bool res = OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 99);
					if (res) break;
					if (ShowAlert) {
						Alert(StringFormat("Closing %s order failed with error %s.", OrderSymbol(), ErrorDescription(GetLastError())));
					}
					Sleep(2000);
				}
			}
		}
	}
}

